@@ -6,7 +6,9 @@
 
 import os
 import sys
+import select
 import logging
+import threading
 from subprocess import (
     call,
     Popen,
@@ -16,27 +18,150 @@ from subprocess import (
 
 from .util import (
     LazyMixin,
-    stream_copy
+    stream_copy,
+    WaitGroup
 )
 from .exc import GitCommandError
 from git.compat import (
     string_types,
     defenc,
-    PY3
+    PY3,
+    # just to satisfy flake8 on py3
+    unicode
 )
 
 execute_kwargs = ('istream', 'with_keep_cwd', 'with_extended_output',
-                  'with_exceptions', 'as_process',
+                  'with_exceptions', 'as_process', 'stdout_as_string',
                   'output_stream')
 
 log = logging.getLogger('git.cmd')
 
 __all__ = ('Git', )
 
+if sys.platform != 'win32':
+    WindowsError = OSError
+
+
+# ==============================================================================
+## @name Utilities
+# ------------------------------------------------------------------------------
+# Documentation
+## @{
+
+def handle_process_output(process, stdout_handler, stderr_handler, finalizer):
+    """Registers for notifications to lean that process output is ready to read, and dispatches lines to
+    the respective line handlers. We are able to handle carriage returns in case progress is sent by that
+    mean. For performance reasons, we only apply this to stderr.
+    This function returns once the finalizer returns
+    :return: result of finalizer
+    :param process: subprocess.Popen instance
+    :param stdout_handler: f(stdout_line_string), or None
+    :param stderr_hanlder: f(stderr_line_string), or None
+    :param finalizer: f(proc) - wait for proc to finish"""
+    def read_line_fast(stream):
+        return stream.readline()
+
+    def read_line_slow(stream):
+        line = b''
+        while True:
+            char = stream.read(1)       # reads individual single byte strings
+            if not char:
+                break
+
+            if char in (b'\r', b'\n') and line:
+                break
+            else:
+                line += char
+            # END process parsed line
+        # END while file is not done reading
+        return line
+    # end
+
+    def dispatch_line(fno):
+            stream, handler, readline = fdmap[fno]
+            # this can possibly block for a while, but since we wake-up with at least one or more lines to handle,
+            # we are good ...
+            line = readline(stream).decode(defenc)
+            if line and handler:
+                try:
+                    handler(line)
+                except Exception:
+                    # Keep reading, have to pump the lines empty nontheless
+                    log.error("Line handler exception on line: %s", line, exc_info=True)
+                # end
+            return line
+        # end dispatch helper
+    # end
+
+    def deplete_buffer(fno, wg=None):
+        while True:
+            line = dispatch_line(fno)
+            if not line:
+                break
+        # end deplete buffer
+        if wg:
+            wg.done()
+    # end
+
+    fdmap = {process.stdout.fileno(): (process.stdout, stdout_handler, read_line_fast),
+             process.stderr.fileno(): (process.stderr, stderr_handler, read_line_slow)}
+
+    if hasattr(select, 'poll'):
+        # poll is preferred, as select is limited to file handles up to 1024 ... . This could otherwise be
+        # an issue for us, as it matters how many handles or own process has
+        poll = select.poll()
+        READ_ONLY = select.POLLIN | select.POLLPRI | select.POLLHUP | select.POLLERR
+        CLOSED = select.POLLHUP | select.POLLERR
+
+        poll.register(process.stdout, READ_ONLY)
+        poll.register(process.stderr, READ_ONLY)
+
+        closed_streams = set()
+        while True:
+            # no timeout
+            poll_result = poll.poll()
+            for fd, result in poll_result:
+                if result & CLOSED:
+                    closed_streams.add(fd)
+                else:
+                    dispatch_line(fd)
+                # end handle closed stream
+            # end for each poll-result tuple
+
+            if len(closed_streams) == len(fdmap):
+                break
+            # end its all done
+        # end endless loop
+
+        # Depelete all remaining buffers
+        for fno in fdmap.keys():
+            deplete_buffer(fno)
+        # end for each file handle
+    else:
+        # Oh ... probably we are on windows. select.select() can only handle sockets, we have files
+        # The only reliable way to do this now is to use threads and wait for both to finish
+        # Since the finalizer is expected to wait, we don't have to introduce our own wait primitive
+        # NO: It's not enough unfortunately, and we will have to sync the threads
+        wg = WaitGroup()
+        for fno in fdmap.keys():
+            wg.add(1)
+            t = threading.Thread(target=lambda: deplete_buffer(fno, wg))
+            t.start()
+        # end
+        # NOTE: Just joining threads can possibly fail as there is a gap between .start() and when it's
+        # actually started, which could make the wait() call to just return because the thread is not yet
+        # active
+        wg.wait()
+    # end
+
+    return finalizer(process)
+
 
 def dashify(string):
     return string.replace('_', '-')
 
+## -- End Utilities -- @}
+
 
 class Git(LazyMixin):
 
@@ -86,11 +211,18 @@ class Git(LazyMixin):
             self.args = args
 
         def __del__(self):
-            self.proc.stdout.close()
-            self.proc.stderr.close()
+            if self.proc is None:
+                return
+
+            proc = self.proc
+            self.proc = None
+            if proc.stdin:
+                proc.stdin.close()
+            proc.stdout.close()
+            proc.stderr.close()
 
             # did the process finish already so we have a return code ?
-            if self.proc.poll() is not None:
+            if proc.poll() is not None:
                 return
 
             # can be that nothing really exists anymore ...
@@ -99,16 +231,16 @@ class Git(LazyMixin):
 
             # try to kill it
             try:
-                os.kill(self.proc.pid, 2)   # interrupt signal
-                self.proc.wait()    # ensure process goes away
-            except OSError:
+                os.kill(proc.pid, 2)   # interrupt signal
+                proc.wait()    # ensure process goes away
+            except (OSError, WindowsError):
                 pass  # ignore error when process already died
             except AttributeError:
                 # try windows
                 # for some reason, providing None for stdout/stderr still prints something. This is why
                 # we simply use the shell and redirect to nul. Its slower than CreateProcess, question
                 # is whether we really want to see all these messages. Its annoying no matter what.
-                call(("TASKKILL /F /T /PID %s 2>nul 1>nul" % str(self.proc.pid)), shell=True)
+                call(("TASKKILL /F /T /PID %s 2>nul 1>nul" % str(proc.pid)), shell=True)
             # END exception handling
 
         def __getattr__(self, attr):
@@ -279,6 +411,7 @@ class Git(LazyMixin):
                 with_exceptions=True,
                 as_process=False,
                 output_stream=None,
+                stdout_as_string=True,
                 **subprocess_kwargs
                 ):
         """Handles executing the command on the shell and consumes and returns
@@ -322,6 +455,11 @@ class Git(LazyMixin):
             output pipe to the given output stream directly.
             Judging from the implementation, you shouldn't use this flag !
 
+        :param stdout_as_string:
+            if False, the commands standard output will be bytes. Otherwise, it will be
+            decoded into a string using the default encoding (usually utf-8).
+            The latter can fail, if the output contains binary data.
+
         :param subprocess_kwargs:
             Keyword arguments to be passed to subprocess.Popen. Please note that
             some of the valid kwargs are already set by this method, the ones you
@@ -364,7 +502,7 @@ class Git(LazyMixin):
                      # Prevent cmd prompt popups on windows by using a shell ... .
                      # See https://github.com/gitpython-developers/GitPython/pull/126
                      shell=sys.platform == 'win32',
-                     close_fds=(os.name == 'posix'),  # unsupported on linux
+                     close_fds=(os.name == 'posix'),  # unsupported on windows
                      **subprocess_kwargs
                      )
         if as_process:
@@ -413,7 +551,7 @@ class Git(LazyMixin):
             else:
                 raise GitCommandError(command, status, stderr_value)
 
-        if isinstance(stdout_value, bytes):  # could also be output_stream
+        if isinstance(stdout_value, bytes) and stdout_as_string:  # could also be output_stream
             stdout_value = stdout_value.decode(defenc)
 
         # Allow access to the command's status code
