@@ -10,7 +10,10 @@ from gitdb.util import hex_to_bin
 from .objects.blob import Blob
 from .objects.util import mode_str_to_int
 
-from git.compat import defenc
+from git.compat import (
+    defenc,
+    PY3
+)
 
 
 __all__ = ('Diffable', 'DiffIndex', 'Diff')
@@ -73,9 +76,9 @@ class Diffable(object):
         args.append("--abbrev=40")        # we need full shas
         args.append("--full-index")       # get full index paths, not only filenames
 
+        args.append("-M")                 # check for renames, in both formats
         if create_patch:
             args.append("-p")
-            args.append("-M")               # check for renames
         else:
             args.append("--raw")
 
@@ -183,8 +186,8 @@ class Diff(object):
 
     # precompiled regex
     re_header = re.compile(r"""
-                                #^diff[ ]--git
-                                    [ ]a/(?P<a_path>.+?)[ ]b/(?P<b_path>.+?)\n
+                                ^diff[ ]--git
+                                    [ ](?:a/)?(?P<a_path>.+?)[ ](?:b/)?(?P<b_path>.+?)\n
                                 (?:^similarity[ ]index[ ](?P<similarity_index>\d+)%\n
                                    ^rename[ ]from[ ](?P<rename_from>\S+)\n
                                    ^rename[ ]to[ ](?P<rename_to>\S+)(?:\n|$))?
@@ -194,7 +197,7 @@ class Diff(object):
                                 (?:^deleted[ ]file[ ]mode[ ](?P<deleted_file_mode>.+)(?:\n|$))?
                                 (?:^index[ ](?P<a_blob_id>[0-9A-Fa-f]+)
                                     \.\.(?P<b_blob_id>[0-9A-Fa-f]+)[ ]?(?P<b_mode>.+)?(?:\n|$))?
-                            """, re.VERBOSE | re.MULTILINE)
+                            """.encode('ascii'), re.VERBOSE | re.MULTILINE)
     # can be used for comparisons
     NULL_HEX_SHA = "0" * 40
     NULL_BIN_SHA = b"\0" * 20
@@ -217,10 +220,12 @@ class Diff(object):
         if a_blob_id is None:
             self.a_blob = None
         else:
+            assert self.a_mode
             self.a_blob = Blob(repo, hex_to_bin(a_blob_id), mode=self.a_mode, path=a_path)
         if b_blob_id is None:
             self.b_blob = None
         else:
+            assert self.b_mode
             self.b_blob = Blob(repo, hex_to_bin(b_blob_id), mode=self.b_mode, path=b_path)
 
         self.new_file = new_file
@@ -278,11 +283,21 @@ class Diff(object):
             msg += '\nfile renamed to %r' % self.rename_to
         if self.diff:
             msg += '\n---'
-            msg += self.diff
+            try:
+                msg += self.diff.decode(defenc)
+            except UnicodeDecodeError:
+                msg += 'OMITTED BINARY DATA'
+            # end handle encoding
             msg += '\n---'
         # END diff info
 
-        return h + msg
+        # Python2 sillyness: have to assure we convert our likely to be unicode object to a string with the
+        # right encoding. Otherwise it tries to convert it using ascii, which may fail ungracefully
+        res = h + msg
+        if not PY3:
+            res = res.encode(defenc)
+        # end
+        return res
 
     @property
     def renamed(self):
@@ -296,35 +311,53 @@ class Diff(object):
         :param stream: result of 'git diff' as a stream (supporting file protocol)
         :return: git.DiffIndex """
         # for now, we have to bake the stream
-        text = stream.read().decode(defenc)
+        text = stream.read()
         index = DiffIndex()
-
-        diff_header = cls.re_header.match
-        for diff in ('\n' + text).split('\ndiff --git')[1:]:
-            header = diff_header(diff)
-
+        previous_header = None
+        for header in cls.re_header.finditer(text):
             a_path, b_path, similarity_index, rename_from, rename_to, \
                 old_mode, new_mode, new_file_mode, deleted_file_mode, \
                 a_blob_id, b_blob_id, b_mode = header.groups()
             new_file, deleted_file = bool(new_file_mode), bool(deleted_file_mode)
 
-            index.append(Diff(repo, a_path, b_path, a_blob_id, b_blob_id,
-                              old_mode or deleted_file_mode, new_mode or new_file_mode or b_mode,
-                              new_file, deleted_file, rename_from, rename_to, diff[header.end():]))
+            # Our only means to find the actual text is to see what has not been matched by our regex,
+            # and then retro-actively assin it to our index
+            if previous_header is not None:
+                index[-1].diff = text[previous_header.end():header.start()]
+            # end assign actual diff
+
+            # Make sure the mode is set if the path is set. Otherwise the resulting blob is invalid
+            # We just use the one mode we should have parsed
+            a_mode = old_mode or deleted_file_mode or (a_path and (b_mode or new_mode or new_file_mode))
+            b_mode = b_mode or new_mode or new_file_mode or (b_path and a_mode)
+            index.append(Diff(repo,
+                              a_path and a_path.decode(defenc),
+                              b_path and b_path.decode(defenc),
+                              a_blob_id and a_blob_id.decode(defenc),
+                              b_blob_id and b_blob_id.decode(defenc),
+                              a_mode and a_mode.decode(defenc),
+                              b_mode and b_mode.decode(defenc),
+                              new_file, deleted_file,
+                              rename_from and rename_from.decode(defenc),
+                              rename_to and rename_to.decode(defenc),
+                              None))
+
+            previous_header = header
+        # end for each header we parse
+        if index:
+            index[-1].diff = text[header.end():]
+        # end assign last diff
 
         return index
 
     @classmethod
     def _index_from_raw_format(cls, repo, stream):
         """Create a new DiffIndex from the given stream which must be in raw format.
-        :note:
-            This format is inherently incapable of detecting renames, hence we only
-            modify, delete and add files
         :return: git.DiffIndex"""
         # handles
         # :100644 100644 687099101... 37c5e30c8... M    .gitignore
         index = DiffIndex()
-        for line in stream:
+        for line in stream.readlines():
             line = line.decode(defenc)
             if not line.startswith(":"):
                 continue
@@ -335,6 +368,8 @@ class Diff(object):
             b_path = path
             deleted_file = False
             new_file = False
+            rename_from = None
+            rename_to = None
 
             # NOTE: We cannot conclude from the existance of a blob to change type
             # as diffs with the working do not have blobs yet
@@ -344,10 +379,13 @@ class Diff(object):
             elif change_type == 'A':
                 a_blob_id = None
                 new_file = True
+            elif change_type[0] == 'R':     # parses RXXX, where XXX is a confidence value
+                a_path, b_path = path.split('\t', 1)
+                rename_from, rename_to = a_path, b_path
             # END add/remove handling
 
             diff = Diff(repo, a_path, b_path, a_blob_id, b_blob_id, old_mode, new_mode,
-                        new_file, deleted_file, None, None, '')
+                        new_file, deleted_file, rename_from, rename_to, '')
             index.append(diff)
         # END for each line
 
