@@ -63,7 +63,8 @@ from .fun import (
     aggressive_tree_merge,
     write_tree_from_cache,
     stat_mode_to_index_mode,
-    S_IFGITLINK
+    S_IFGITLINK,
+    run_commit_hook
 )
 
 from gitdb.base import IStream
@@ -287,11 +288,11 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             changes according to the amount of trees.
             If 1 Tree is given, it will just be read into a new index
             If 2 Trees are given, they will be merged into a new index using a
-             two way merge algorithm. Tree 1 is the 'current' tree, tree 2 is the 'other'
-             one. It behaves like a fast-forward.
-             If 3 Trees are given, a 3-way merge will be performed with the first tree
-             being the common ancestor of tree 2 and tree 3. Tree 2 is the 'current' tree,
-             tree 3 is the 'other' one
+            two way merge algorithm. Tree 1 is the 'current' tree, tree 2 is the 'other'
+            one. It behaves like a fast-forward.
+            If 3 Trees are given, a 3-way merge will be performed with the first tree
+            being the common ancestor of tree 2 and tree 3. Tree 2 is the 'current' tree,
+            tree 3 is the 'other' one
 
         :param kwargs:
             Additional arguments passed to git-read-tree
@@ -579,8 +580,14 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         entries_added = list()
         if path_rewriter:
             for path in paths:
-                abspath = os.path.abspath(path)
-                gitrelative_path = abspath[len(self.repo.working_tree_dir) + 1:]
+                if os.path.isabs(path):
+                    abspath = path
+                    gitrelative_path = path[len(self.repo.working_tree_dir) + 1:]
+                else:
+                    gitrelative_path = path
+                    abspath = os.path.join(self.repo.working_tree_dir, gitrelative_path)
+                # end obtain relative and absolute paths
+
                 blob = Blob(self.repo, Blob.NULL_BIN_SHA,
                             stat_mode_to_index_mode(os.stat(abspath).st_mode),
                             to_native_path_linux(gitrelative_path))
@@ -882,17 +889,17 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
     def commit(self, message, parent_commits=None, head=True, author=None, committer=None):
         """Commit the current default index file, creating a commit object.
-
         For more information on the arguments, see tree.commit.
-        :note:
-            If you have manually altered the .entries member of this instance,
-            don't forget to write() your changes to disk beforehand.
 
-        :return:
-            Commit object representing the new commit"""
+        :note: If you have manually altered the .entries member of this instance,
+               don't forget to write() your changes to disk beforehand.
+        :return: Commit object representing the new commit"""
+        run_commit_hook('pre-commit', self)
         tree = self.write_tree()
-        return Commit.create_from_tree(self.repo, tree, message, parent_commits,
+        rval = Commit.create_from_tree(self.repo, tree, message, parent_commits,
                                        head, author=author, committer=committer)
+        run_commit_hook('post-commit', self)
+        return rval
 
     @classmethod
     def _flush_stdin_and_wait(cls, proc, ignore_stdout=False):
