@@ -46,17 +46,17 @@ def get_object_type_by_name(object_type_name):
     :param object_type_name: Member of TYPES
 
     :raise ValueError: In case object_type_name is unknown"""
-    if object_type_name == "commit":
-        import commit
+    if object_type_name == b"commit":
+        from . import commit
         return commit.Commit
-    elif object_type_name == "tag":
-        import tag
+    elif object_type_name == b"tag":
+        from . import tag
         return tag.TagObject
-    elif object_type_name == "blob":
-        import blob
+    elif object_type_name == b"blob":
+        from . import blob
         return blob.Blob
-    elif object_type_name == "tree":
-        import tree
+    elif object_type_name == b"tree":
+        from . import tree
         return tree.Tree
     else:
         raise ValueError("Cannot handle unknown object type: %s" % object_type_name)
