@@ -5,7 +5,10 @@
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 
 from git.exc import InvalidGitRepositoryError, NoSuchPathError
-from git.cmd import Git
+from git.cmd import (
+    Git,
+    handle_process_output
+)
 from git.refs import (
     HEAD,
     Head,
@@ -25,7 +28,6 @@ from git.index import IndexFile
 from git.config import GitConfigParser
 from git.remote import (
     Remote,
-    digest_process_messages,
     add_progress
 )
 
@@ -44,7 +46,6 @@ from .fun import (
     rev_parse,
     is_git_dir,
     find_git_dir,
-    read_gitfile,
     touch,
 )
 from git.compat import (
@@ -57,7 +58,7 @@ import sys
 import re
 
 DefaultDBType = GitDB
-if sys.version_info[1] < 5:     # python 2.4 compatiblity
+if sys.version_info[:2] < (2, 5):     # python 2.4 compatiblity
     DefaultDBType = GitCmdObjectDB
 # END handle python 2.4
 
@@ -65,6 +66,10 @@ if sys.version_info[1] < 5:     # python 2.4 compatiblity
 __all__ = ('Repo', )
 
 
+def _expand_path(p):
+    return os.path.abspath(os.path.expandvars(os.path.expanduser(p)))
+
+
 class Repo(object):
 
     """Represents a git repository and allows you to query references,
@@ -92,9 +97,9 @@ class Repo(object):
 
     # invariants
     # represents the configuration level of a configuration file
-    config_level = ("system", "global", "repository")
+    config_level = ("system", "user", "global", "repository")
 
-    def __init__(self, path=None, odbt=DefaultDBType):
+    def __init__(self, path=None, odbt=DefaultDBType, search_parent_directories=False):
         """Create a new Repo instance
 
         :param path: is the path to either the root git directory or the bare git repo::
@@ -110,8 +115,8 @@ class Repo(object):
         :raise InvalidGitRepositoryError:
         :raise NoSuchPathError:
         :return: git.Repo """
-        epath = os.path.abspath(os.path.expandvars(os.path.expanduser(path or os.getcwd())))
-
+        epath = _expand_path(path or os.getcwd())
+        self.git = None  # should be set for __del__ not to fail in case we raise
         if not os.path.exists(epath):
             raise NoSuchPathError(epath)
 
@@ -124,17 +129,16 @@ class Repo(object):
         while curpath:
             if is_git_dir(curpath):
                 self.git_dir = curpath
-                self._working_tree_dir = os.path.dirname(curpath)
+                self._working_tree_dir = os.path.dirname(self.git_dir)
                 break
+
             gitpath = find_git_dir(join(curpath, '.git'))
             if gitpath is not None:
                 self.git_dir = gitpath
                 self._working_tree_dir = curpath
                 break
-            gitpath = read_gitfile(curpath)
-            if gitpath:
-                self.git_dir = gitpath
-                self._working_tree_dir = curpath
+
+            if not search_parent_directories:
                 break
             curpath, dummy = os.path.split(curpath)
             if not dummy:
@@ -166,6 +170,10 @@ class Repo(object):
             args.append(self.git)
         self.odb = odbt(*args)
 
+    def __del__(self):
+        if self.git:
+            self.git.clear_cache()
+
     def __eq__(self, rhs):
         if isinstance(rhs, Repo):
             return self.git_dir == rhs.git_dir
@@ -244,7 +252,10 @@ class Repo(object):
     def remote(self, name='origin'):
         """:return: Remote with the specified name
         :raise ValueError:  if no remote with such a name exists"""
-        return Remote(self, name)
+        r = Remote(self, name)
+        if not r.exists():
+            raise ValueError("Remote named '%s' didn't exist" % name)
+        return r
 
     #{ Submodules
 
@@ -342,6 +353,9 @@ class Repo(object):
 
         if config_level == "system":
             return "/etc/gitconfig"
+        elif config_level == "user":
+            config_home = os.environ.get("XDG_CONFIG_HOME") or os.path.join(os.environ.get("HOME", '~'), ".config")
+            return os.path.expanduser(join(config_home, "git", "config"))
         elif config_level == "global":
             return os.path.normpath(os.path.expanduser("~/.gitconfig"))
         elif config_level == "repository":
@@ -578,14 +592,28 @@ class Repo(object):
             A list of tuples associating a Commit object with a list of lines that
             changed within the given commit. The Commit objects will be given in order
             of appearance."""
-        data = self.git.blame(rev, '--', file, p=True)
+        data = self.git.blame(rev, '--', file, p=True, stdout_as_string=False)
         commits = dict()
         blames = list()
         info = None
 
-        for line in data.splitlines(False):
-            parts = self.re_whitespace.split(line, 1)
-            firstpart = parts[0]
+        keepends = True
+        for line in data.splitlines(keepends):
+            try:
+                line = line.rstrip().decode(defenc)
+            except UnicodeDecodeError:
+                firstpart = ''
+                is_binary = True
+            else:
+                # As we don't have an idea when the binary data ends, as it could contain multiple newlines
+                # in the process. So we rely on being able to decode to tell us what is is.
+                # This can absolutely fail even on text files, but even if it does, we should be fine treating it
+                # as binary instead
+                parts = self.re_whitespace.split(line, 1)
+                firstpart = parts[0]
+                is_binary = False
+            # end handle decode of line
+
             if self.re_hexsha_only.search(firstpart):
                 # handles
                 # 634396b2f541a9f2d58b00be1a07f0c358b999b3 1 1 7        - indicates blame-data start
@@ -642,10 +670,18 @@ class Repo(object):
                                            message=info['summary'])
                                 commits[sha] = c
                             # END if commit objects needs initial creation
-                            m = self.re_tab_full_line.search(line)
-                            text, = m.groups()
+                            if not is_binary:
+                                if line and line[0] == '\t':
+                                    line = line[1:]
+                            else:
+                                # NOTE: We are actually parsing lines out of binary data, which can lead to the
+                                # binary being split up along the newline separator. We will append this to the blame
+                                # we are currently looking at, even though it should be concatenated with the last line
+                                # we have seen.
+                                pass
+                            # end handle line contents
                             blames[-1][0] = c
-                            blames[-1][1].append(text)
+                            blames[-1][1].append(line)
                             info = {'id': sha}
                         # END if we collected commit info
                     # END distinguish filename,summary,rest
@@ -671,7 +707,8 @@ class Repo(object):
             keyword arguments serving as additional options to the git-init command
 
         :return: ``git.Repo`` (the newly created repo)"""
-
+        if path:
+            path = _expand_path(path)
         if mkdir and path and not os.path.exists(path):
             os.makedirs(path, 0o755)
 
@@ -711,9 +748,10 @@ class Repo(object):
             proc = git.clone(url, path, with_extended_output=True, as_process=True,
                              v=True, **add_progress(kwargs, git, progress))
             if progress:
-                digest_process_messages(proc.stderr, progress)
-            # END handle progress
-            finalize_process(proc)
+                handle_process_output(proc, None, progress.new_message_handler(), finalize_process)
+            else:
+                finalize_process(proc)
+            # end handle progress
         finally:
             if prev_cwd is not None:
                 os.chdir(prev_cwd)
