@@ -2,7 +2,10 @@
 import os
 from string import digits
 
-from gitdb.exc import BadObject
+from gitdb.exc import (
+    BadObject,
+    BadName
+)
 from git.refs import SymbolicReference
 from git.objects import Object
 from gitdb.util import (
@@ -16,8 +19,8 @@ from gitdb.util import (
 from git.compat import xrange
 
 
-__all__ = ('rev_parse', 'is_git_dir', 'touch', 'read_gitfile', 'find_git_dir', 'name_to_object',
-           'short_to_long', 'deref_tag', 'to_commit')
+__all__ = ('rev_parse', 'is_git_dir', 'touch', 'find_git_dir', 'name_to_object', 'short_to_long', 'deref_tag',
+           'to_commit')
 
 
 def touch(filename):
@@ -41,32 +44,23 @@ def is_git_dir(d):
 def find_git_dir(d):
     if is_git_dir(d):
         return d
-    elif isfile(d):
+
+    try:
         with open(d) as fp:
             content = fp.read().rstrip()
+    except (IOError, OSError):
+        # it's probably not a file
+        pass
+    else:
         if content.startswith('gitdir: '):
-            d = join(dirname(d), content[8:])
-            return find_git_dir(d)
+            path = content[8:]
+            if not os.path.isabs(path):
+                path = join(dirname(d), path)
+            return find_git_dir(path)
+    # end handle exception
     return None
 
 
-def read_gitfile(f):
-    """ This is taken from the git setup.c:read_gitfile function.
-    :return gitdir path or None if gitfile is invalid."""
-    if f is None:
-        return None
-    try:
-        line = open(f, 'r').readline().rstrip()
-    except (OSError, IOError):
-        # File might not exist or is unreadable - ignore
-        return None
-    # end handle file access
-    if line[0:8] != 'gitdir: ':
-        return None
-    path = os.path.realpath(line[8:])
-    return path if is_git_dir(path) else None
-
-
 def short_to_long(odb, hexsha):
     """:return: long hexadecimal sha1 from the given less-than-40 byte hexsha
         or None if no candidate could be found.
@@ -83,7 +77,7 @@ def name_to_object(repo, name, return_ref=False):
     :return: object specified by the given name, hexshas ( short and long )
         as well as references are supported
     :param return_ref: if name specifies a reference, we will return the reference
-        instead of the object. Otherwise it will raise BadObject
+        instead of the object. Otherwise it will raise BadObject or BadName
     """
     hexsha = None
 
@@ -119,7 +113,7 @@ def name_to_object(repo, name, return_ref=False):
 
     # tried everything ? fail
     if hexsha is None:
-        raise BadObject(name)
+        raise BadName(name)
     # END assert hexsha was found
 
     return Object.new_from_sha(repo, hex_to_bin(hexsha))
