@@ -62,10 +62,14 @@ class StringProcessAdapter(object):
 
 
 def _mktemp(*args):
-    """Wrapper around default tempfile.mktemp to fix an osx issue"""
+    """Wrapper around default tempfile.mktemp to fix an osx issue
+    :note: the OSX special case was removed as it was unclear why that was needed in the first place. It seems
+    to be just fine without it. However, if we leave this special case, and if TMPDIR is set to something custom,
+    prefixing /private/ will lead to incorrect paths on OSX."""
     tdir = tempfile.mktemp(*args)
-    if sys.platform == 'darwin':
-        tdir = '/private' + tdir
+    # See :note: above to learn why this is comented out.
+    # if sys.platform == 'darwin':
+    #     tdir = '/private' + tdir
     return tdir
 
 
@@ -279,6 +283,11 @@ class TestBase(TestCase):
         """
         cls.rorepo = Repo(GIT_REPO)
 
+    @classmethod
+    def tearDownClass(cls):
+        cls.rorepo.git.clear_cache()
+        cls.rorepo.git = None
+
     def _make_file(self, rela_path, data, repo=None):
         """
         Create a file at the given path relative to our repository, filled
