@@ -6,7 +6,8 @@
 
 from git.test.lib import (
     TestCase,
-    fixture_path
+    fixture_path,
+    assert_equal
 )
 from git import (
     GitConfigParser
@@ -72,6 +73,24 @@ class TestBase(TestCase):
             assert r_config.get(sname, oname) == val
         # END for each filename
 
+    def test_multi_line_config(self):
+        file_obj = self._to_memcache(fixture_path("git_config_with_comments"))
+        config = GitConfigParser(file_obj, read_only=False)
+        ev = "ruby -e '\n"
+        ev += "		system %(git), %(merge-file), %(--marker-size=%L), %(%A), %(%O), %(%B)\n"
+        ev += "		b = File.read(%(%A))\n"
+        ev += "		b.sub!(/^<+ .*\\nActiveRecord::Schema\\.define.:version => (\\d+). do\\n=+\\nActiveRecord::Schema\\."
+        ev += "define.:version => (\\d+). do\\n>+ .*/) do\n"
+        ev += "		  %(ActiveRecord::Schema.define(:version => #{[$1, $2].max}) do)\n"
+        ev += "		end\n"
+        ev += "		File.open(%(%A), %(w)) {|f| f.write(b)}\n"
+        ev += "		exit 1 if b.include?(%(<)*%L)'"
+        assert_equal(config.get('merge "railsschema"', 'driver'), ev)
+        assert_equal(config.get('alias', 'lg'),
+                     "log --graph --pretty=format:'%Cred%h%Creset -%C(yellow)%d%Creset %s %Cgreen(%cr)%Creset'"
+                     " --abbrev-commit --date=relative")
+        assert len(config.sections()) == 23
+
     def test_base(self):
         path_repo = fixture_path("git_config")
         path_global = fixture_path("git_config_global")
