@@ -1,3 +1,4 @@
+#-*-coding:utf-8-*-
 # test_index.py
 # Copyright (C) 2008, 2009 Michael Trier (mtrier@gmail.com) and contributors
 #
@@ -10,6 +11,8 @@ from git.test.lib import (
     fixture,
     with_rw_repo
 )
+from git.util import Actor
+from git.exc import HookExecutionError
 from git import (
     IndexFile,
     BlobFilter,
@@ -38,6 +41,7 @@ from git.index.typ import (
     BaseIndexEntry,
     IndexEntry
 )
+from git.index.fun import hook_path
 
 
 class TestIndex(TestBase):
@@ -432,7 +436,7 @@ class TestIndex(TestBase):
         # TEST COMMITTING
         # commit changed index
         cur_commit = cur_head.commit
-        commit_message = "commit default head"
+        commit_message = u"commit default head by Frèderic Çaufl€"
 
         new_commit = index.commit(commit_message, head=False)
         assert cur_commit != new_commit
@@ -445,6 +449,23 @@ class TestIndex(TestBase):
         assert len(new_commit.parents) == 1
         assert cur_head.commit == cur_commit
 
+        # commit with other actor
+        cur_commit = cur_head.commit
+
+        my_author = Actor(u"Frèderic Çaufl€", "author@example.com")
+        my_committer = Actor(u"Committing Frèderic Çaufl€", "committer@example.com")
+        commit_actor = index.commit(commit_message, author=my_author, committer=my_committer)
+        assert cur_commit != commit_actor
+        assert commit_actor.author.name == u"Frèderic Çaufl€"
+        assert commit_actor.author.email == "author@example.com"
+        assert commit_actor.committer.name == u"Committing Frèderic Çaufl€"
+        assert commit_actor.committer.email == "committer@example.com"
+        assert commit_actor.message == commit_message
+        assert commit_actor.parents[0] == cur_commit
+        assert len(new_commit.parents) == 1
+        assert cur_head.commit == commit_actor
+        assert cur_head.log()[-1].actor == my_committer
+
         # same index, no parents
         commit_message = "index without parents"
         commit_no_parents = index.commit(commit_message, parent_commits=list(), head=True)
@@ -482,7 +503,7 @@ class TestIndex(TestBase):
 
         # same file
         entries = index.reset(new_commit).add(
-            [os.path.abspath(os.path.join('lib', 'git', 'head.py'))] * 2, fprogress=self._fprogress_add)
+            [os.path.join(rw_repo.working_tree_dir, 'lib', 'git', 'head.py')] * 2, fprogress=self._fprogress_add)
         self._assert_entries(entries)
         assert entries[0].mode & 0o644 == 0o644
         # would fail, test is too primitive to handle this case
@@ -646,6 +667,25 @@ class TestIndex(TestBase):
             assert fkey not in index.entries
 
         index.add(files, write=True)
+        if os.name != 'nt':
+            hp = hook_path('pre-commit', index.repo.git_dir)
+            with open(hp, "wt") as fp:
+                fp.write("#!/usr/bin/env sh\necho stdout; echo stderr 1>&2; exit 1")
+            # end
+            os.chmod(hp, 0o544)
+            try:
+                index.commit("This should fail")
+            except HookExecutionError as err:
+                assert err.status == 1
+                assert err.command == hp
+                assert err.stdout == 'stdout\n'
+                assert err.stderr == 'stderr\n'
+                assert str(err)
+            else:
+                raise AssertionError("Should have cought a HookExecutionError")
+            # end exception handling
+            os.remove(hp)
+        # end hook testing
         nc = index.commit("2 files committed", head=False)
 
         for fkey in keys:
