@@ -23,6 +23,7 @@ from git import (
     GitCommandError
 )
 from git.util import IterableList
+from git.compat import string_types
 import tempfile
 import shutil
 import os
@@ -97,7 +98,7 @@ class TestRemote(TestBase):
         # self._print_fetchhead(remote.repo)
         assert len(results) > 0 and isinstance(results[0], FetchInfo)
         for info in results:
-            assert isinstance(info.note, basestring)
+            assert isinstance(info.note, string_types)
             if isinstance(info.ref, Reference):
                 assert info.flags != 0
             # END reference type flags handling
@@ -113,7 +114,7 @@ class TestRemote(TestBase):
         assert len(results) > 0 and isinstance(results[0], PushInfo)
         for info in results:
             assert info.flags
-            assert isinstance(info.summary, basestring)
+            assert isinstance(info.summary, string_types)
             if info.old_commit is not None:
                 assert isinstance(info.old_commit, Commit)
             if info.flags & info.ERROR:
