@@ -25,15 +25,16 @@ from git import (
     Reference,
     GitDB,
     Submodule,
-    GitCmdObjectDB
+    GitCmdObjectDB,
+    Remote,
+    BadName
 )
+from git.repo.fun import touch
 from git.util import join_path_native
 from git.exc import BadObject
 from gitdb.util import bin_to_hex
-from git.compat import (
-    string_types,
-    defenc
-)
+from git.compat import string_types
+from gitdb.test.lib import with_rw_directory
 
 import os
 import sys
@@ -85,7 +86,7 @@ class TestRepo(TestBase):
         assert self.rorepo.tree(tree) == tree
 
         # try from invalid revision that does not exist
-        self.failUnlessRaises(BadObject, self.rorepo.tree, 'hello world')
+        self.failUnlessRaises(BadName, self.rorepo.tree, 'hello world')
 
     def test_commit_from_revision(self):
         commit = self.rorepo.commit('0.1.4')
@@ -271,7 +272,7 @@ class TestRepo(TestBase):
 
     @patch.object(Git, '_call_process')
     def test_should_display_blame_information(self, git):
-        git.return_value = fixture('blame').decode(defenc)
+        git.return_value = fixture('blame')
         b = self.rorepo.blame('master', 'lib/git.py')
         assert_equal(13, len(b))
         assert_equal(2, len(b[0]))
@@ -279,7 +280,6 @@ class TestRepo(TestBase):
         assert_equal(hash(b[0][0]), hash(b[9][0]))
         c = b[0][0]
         assert_true(git.called)
-        assert_equal(git.call_args, (('blame', 'master', '--', 'lib/git.py'), {'p': True}))
 
         assert_equal('634396b2f541a9f2d58b00be1a07f0c358b999b3', c.hexsha)
         assert_equal('Tom Preston-Werner', c.author.name)
@@ -296,14 +296,30 @@ class TestRepo(TestBase):
         assert_true(isinstance(tlist[0], string_types))
         assert_true(len(tlist) < sum(len(t) for t in tlist))               # test for single-char bug
 
+        # BINARY BLAME
+        git.return_value = fixture('blame_binary')
+        blames = self.rorepo.blame('master', 'rps')
+        assert len(blames) == 2
+
     def test_blame_real(self):
         c = 0
+        nml = 0   # amount of multi-lines per blame
         for item in self.rorepo.head.commit.tree.traverse(
                 predicate=lambda i, d: i.type == 'blob' and i.path.endswith('.py')):
             c += 1
-            self.rorepo.blame(self.rorepo.head, item.path)
+
+            for b in self.rorepo.blame(self.rorepo.head, item.path):
+                nml += int(len(b[1]) > 1)
         # END for each item to traverse
-        assert c
+        assert c, "Should have executed at least one blame command"
+        assert nml, "There should at least be one blame commit that contains multiple lines"
+
+    @patch.object(Git, '_call_process')
+    def test_blame_complex_revision(self, git):
+        git.return_value = fixture('blame_complex_revision')
+        res = self.rorepo.blame("HEAD~10..HEAD", "README.md")
+        assert len(res) == 1
+        assert len(res[0][1]) == 83, "Unexpected amount of parsed blame lines"
 
     def test_untracked_files(self):
         base = self.rorepo.working_tree_dir
@@ -349,6 +365,11 @@ class TestRepo(TestBase):
                 pass
         # END for each config level
 
+    def test_config_level_paths(self):
+        for config_level in self.rorepo.config_level:
+            assert self.rorepo._get_config_path(config_level)
+        # end for each config level
+
     def test_creation_deletion(self):
         # just a very quick test to assure it generally works. There are
         # specialized cases in the test_refs module
@@ -367,6 +388,20 @@ class TestRepo(TestBase):
         assert self.rorepo == self.rorepo and not (self.rorepo != self.rorepo)
         assert len(set((self.rorepo, self.rorepo))) == 1
 
+    @with_rw_directory
+    def test_tilde_and_env_vars_in_repo_path(self, rw_dir):
+        ph = os.environ['HOME']
+        try:
+            os.environ['HOME'] = rw_dir
+            Repo.init(os.path.join('~', 'test.git'), bare=True)
+
+            os.environ['FOO'] = rw_dir
+            Repo.init(os.path.join('$FOO', 'test.git'), bare=True)
+        finally:
+            os.environ['HOME'] = ph
+            del os.environ['FOO']
+        # end assure HOME gets reset to what it was
+
     def test_git_cmd(self):
         # test CatFileContentStream, just to be very sure we have no fencepost errors
         # last \n is the terminating newline that it expects
@@ -525,8 +560,8 @@ class TestRepo(TestBase):
                     obj = self._assert_rev_parse(path_section)
                     assert obj.type == ref.object.type
                     num_resolved += 1
-                except BadObject:
-                    print ("failed on %s" % path_section)
+                except (BadName, BadObject):
+                    print("failed on %s" % path_section)
                     # is fine, in case we have something like 112, which belongs to remotes/rname/merge-requests/112
                     pass
                 # END exception handling
@@ -615,7 +650,7 @@ class TestRepo(TestBase):
 
     def test_repo_odbtype(self):
         target_type = GitDB
-        if sys.version_info[1] < 5:
+        if sys.version_info[:2] < (2, 5):
             target_type = GitCmdObjectDB
         assert isinstance(self.rorepo.odb, target_type)
 
@@ -656,3 +691,49 @@ class TestRepo(TestBase):
         open(git_file_path, 'wb').write(('gitdir: %s\n' % real_path_abs).encode('ascii'))
         git_file_repo = Repo(rwrepo.working_tree_dir)
         assert os.path.abspath(git_file_repo.git_dir) == real_path_abs
+
+    def test_file_handle_leaks(self):
+        def last_commit(repo, rev, path):
+            commit = next(repo.iter_commits(rev, path, max_count=1))
+            commit.tree[path]
+
+        # This is based on this comment
+        # https://github.com/gitpython-developers/GitPython/issues/60#issuecomment-23558741
+        # And we expect to set max handles to a low value, like 64
+        # You should set ulimit -n X, see .travis.yml
+        # The loops below would easily create 500 handles if these would leak (4 pipes + multiple mapped files)
+        for i in range(64):
+            for repo_type in (GitCmdObjectDB, GitDB):
+                repo = Repo(self.rorepo.working_tree_dir, odbt=repo_type)
+                last_commit(repo, 'master', 'git/test/test_base.py')
+            # end for each repository type
+        # end for each iteration
+
+    def test_remote_method(self):
+        self.failUnlessRaises(ValueError, self.rorepo.remote, 'foo-blue')
+        assert isinstance(self.rorepo.remote(name='origin'), Remote)
+
+    @with_rw_directory
+    def test_empty_repo(self, rw_dir):
+        """Assure we can handle empty repositories"""
+        r = Repo.init(rw_dir, mkdir=False)
+        # It's ok not to be able to iterate a commit, as there is none
+        self.failUnlessRaises(ValueError, r.iter_commits)
+        assert r.active_branch.name == 'master'
+        assert not r.active_branch.is_valid(), "Branch is yet to be born"
+
+        # actually, when trying to create a new branch without a commit, git itself fails
+        # We should, however, not fail ungracefully
+        self.failUnlessRaises(BadName, r.create_head, 'foo')
+        self.failUnlessRaises(BadName, r.create_head, 'master')
+        # It's expected to not be able to access a tree
+        self.failUnlessRaises(ValueError, r.tree)
+
+        new_file_path = os.path.join(rw_dir, "new_file.ext")
+        touch(new_file_path)
+        r.index.add([new_file_path])
+        r.index.commit("initial commit")
+
+        # Now a branch should be creatable
+        nb = r.create_head('foo')
+        assert nb.is_valid()
